package announcement_management

import (
	"adm-management/consts"
	"adm-management/errors"
	"adm-management/model"
	v1 "adm-management/pb/v1"
	"context"

	"encoding/json"
	"fmt"

	"git.getensh.com/common/gopkgsv2/database"
	"git.getensh.com/common/gopkgsv2/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
)

func AnnouncementList(ctx context.Context, req *v1.AnnouncementRequest) (reply *v1.AnnouncementReply, err error) {
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	reply = &v1.AnnouncementReply{}

	db := database.DB()
	if req.VinRule == "" {
		db = db.Where("")
	}
	if req.VinRule != "" {
		var modelList []string
		err = db.Raw("SELECT model_no FROM db_adm_dws.t_adm_dws11 where vin_rule = ? ", req.VinRule).Find(&modelList).Error
		if err != nil {
			if err == gorm.ErrRecordNotFound {
				return reply, nil
			}
			return reply, errors.SystemError
		}
		if len(modelList) == 0 {
			db = db.Where("model_no = ?", modelList)
		} else {
			db = db.Where("model_no in (?)", modelList)
		}

	}
	if req.BrandName == "" {
		db = db.Where("brand_name is NOT null")
	}
	if req.BrandName != "" {
		db = db.Where("brand_name = ?", req.BrandName)
	}
	if req.ModelNo == "" {
		db = db.Where("model_no is NOT null")
	}
	if req.ModelNo != "" {
		db = db.Where("model_no = ?", req.ModelNo)
	}
	pageSize := consts.PageSize
	if req.PageSize != 0 {
		pageSize = int(req.PageSize)
	}

	// 构造分页类
	pagination := model.NewPagination(int(req.Page), pageSize, 0).GetLimitOffset()

	count, err := model.NewAnnouncementList().Count(db)
	if err != nil && err != gorm.ErrRecordNotFound {
		return reply, errors.SystemError
	}

	if err == gorm.ErrRecordNotFound || count == 0 {
		return reply, nil
	}

	list, err := model.NewAnnouncementList().List(db, pagination)

	if err != nil && err != gorm.ErrRecordNotFound {
		return reply, errors.SystemError
	}

	if err == gorm.ErrRecordNotFound {
		pagination.CalPage(int(count))

		reply.CurrentPage = int64(pagination.CurrentPage)
		reply.PerPage = int64(pagination.PerPage)
		reply.Total = int64(pagination.Total)
		reply.FirstPage = int64(pagination.FirstPage)
		reply.LastPage = int64(pagination.LastPage)
		reply.PrevPage = int64(pagination.PrevPage)
		reply.NextPage = int64(pagination.NextPage)
		return reply, nil
	}

	reply.List = make([]*v1.AnnouncementList, 0, len(list))
	for _, v := range list {
		reply.List = append(reply.List, &v1.AnnouncementList{
			Id:                v.Id,
			VehicleBatch:      v.VehicleBatch,
			VinRules:          v.VinRules,
			ModelNo:           v.ModelNo,
			BrandName:         v.BrandName,
			VehicleTypeDetail: v.VehicleTypeDetail,
		})
	}

	// 计算分页结果
	pagination.CalPage(int(count))

	reply.CurrentPage = int64(pagination.CurrentPage)
	reply.PerPage = int64(pagination.PerPage)
	reply.Total = int64(pagination.Total)
	reply.FirstPage = int64(pagination.FirstPage)
	reply.LastPage = int64(pagination.LastPage)
	reply.PrevPage = int64(pagination.PrevPage)
	reply.NextPage = int64(pagination.NextPage)
	return reply, nil
}
