package data_management

import (
	"context"
	"encoding/json"
	"fmt"

	"adm-management/errors"
	"adm-management/model"
	v1 "adm-management/pb/v1"

	"git.getensh.com/common/gopkgsv2/database"
	"git.getensh.com/common/gopkgsv2/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
)

func SearchData(ctx context.Context, req *v1.SearchDataRequest) (reply *v1.SearchDataReply, err error) {
	reply = &v1.SearchDataReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.TableName != "" {
		// 表名搜索
		// 构造分页类
		pagination := model.NewPagination(0, 100000, 0).GetLimitOffset()
		list, err := model.NewDataList().List(database.DB().Where("table_name like ?", "%"+req.TableName+"%"), pagination)
		if err != nil && err != gorm.ErrRecordNotFound {
			return reply, errors.SystemError
		}

		if err == gorm.ErrRecordNotFound {
			return reply, nil
		}

		reply.List = make([]*v1.SearchData, 0, len(list))
		for _, v := range list {
			reply.List = append(reply.List, &v1.SearchData{
				TableName: v.TableName,
				Desc:      v.Desc,
			})
		}
	}

	if req.TableName == "" {
		// 表名搜索
		// 构造分页类
		pagination := model.NewPagination(0, 100000, 0).GetLimitOffset()
		list, err := model.NewDataList().List(database.DB().Where("table_name is NOT null"), pagination)
		if err != nil && err != gorm.ErrRecordNotFound {
			return reply, errors.SystemError
		}

		if err == gorm.ErrRecordNotFound {
			return reply, nil
		}

		reply.List = make([]*v1.SearchData, 0, len(list))
		for _, v := range list {
			reply.List = append(reply.List, &v1.SearchData{
				TableName: v.TableName,
				Desc:      v.Desc,
			})
		}
	}

	return reply, nil
}

func SearchLevel(ctx context.Context, req *v1.SearchLevelRequest) (reply *v1.SearchLevelReply, err error) {
	reply = &v1.SearchLevelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.Level != "" {

		db := database.DB()
		var levelList []string
		err = db.Raw("SELECT DISTINCT `level` from t_adm_data_management where `level` like ?", "%"+req.Level+"%").Find(&levelList).Error
		if err != nil {
			if err == gorm.ErrRecordNotFound {
				return reply, nil
			}
			return reply, errors.SystemError
		}

		reply.List = make([]string, 0, len(levelList))
		for _, v := range levelList {
			reply.List = append(reply.List, v)
		}
	}

	if req.Level == "" {

		db := database.DB()
		var levelList []string
		err = db.Raw("SELECT DISTINCT `level` from t_adm_data_management where `level` is not null").Find(&levelList).Error
		if err != nil {
			if err == gorm.ErrRecordNotFound {
				return reply, nil
			}
			return reply, errors.SystemError
		}

		reply.List = make([]string, 0, len(levelList))
		for _, v := range levelList {
			reply.List = append(reply.List, v)
		}
	}

	return reply, nil
}
