package manual_task

import (
	"adm-management/consts"
	"adm-management/errors"
	"adm-management/model"
	v1 "adm-management/pb/v1"
	"math"
	"strconv"
	"time"

	"context"

	"encoding/json"
	"fmt"
	"strings"

	"git.getensh.com/common/gopkgsv2/database"
	"git.getensh.com/common/gopkgsv2/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
)

func ManualTask(ctx context.Context, req *v1.ManualTaskRequest) (reply *v1.ManualTaskReply, err error) {
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	reply = &v1.ManualTaskReply{}

	db := database.DB()
	if req.Source == "" {
		db = db.Where("source is NOT null")
	}
	if req.Source != "" {
		db = db.Where("source = ?", req.Source)
	}
	if req.TaskName == "" {
		db = db.Where("task_ids is NOT null")
	}
	if req.TaskName != "" {
		task, err := model.NewTaskList().Get(database.DB().Where("task_name = ?", req.TaskName))
		if err != nil && err != gorm.ErrRecordNotFound {
			return nil, err
		}
		if err == gorm.ErrRecordNotFound {
			return reply, nil
		}
		db = db.Where("task_ids like ?", "%"+strconv.Itoa(int(task.TaskId))+"%")
	}
	if req.ManualTaskName == "" {
		db = db.Where("task_name is NOT null")
	}
	if req.ManualTaskName != "" {
		db = db.Where("task_name like ?", "%"+req.ManualTaskName+"%")
	}
	if req.Time1 == 0 && req.Time2 == 0 {
		db = db.Where("created_at is NOT null")
	} else {
		db = db.Where("created_at >= ? and created_at <= ?", req.Time1, req.Time2)
	}

	db = db.Order("created_at desc")

	pageSize := consts.PageSize
	if req.PageSize != 0 {
		pageSize = int(req.PageSize)
	}

	// 构造分页类
	pagination := model.NewPagination(int(req.Page), pageSize, 0).GetLimitOffset()

	count, err := model.NewManualTask().Count(db)
	if err != nil && err != gorm.ErrRecordNotFound {
		return reply, errors.SystemError
	}

	if err == gorm.ErrRecordNotFound || count == 0 {
		return reply, nil
	}

	list, err := model.NewManualTask().List(db, pagination)

	if err != nil && err != gorm.ErrRecordNotFound {
		return reply, errors.SystemError
	}

	if err == gorm.ErrRecordNotFound {
		pagination.CalPage(int(count))

		reply.CurrentPage = int64(pagination.CurrentPage)
		reply.PerPage = int64(pagination.PerPage)
		reply.Total = int64(pagination.Total)
		reply.FirstPage = int64(pagination.FirstPage)
		reply.LastPage = int64(pagination.LastPage)
		reply.PrevPage = int64(pagination.PrevPage)
		reply.NextPage = int64(pagination.NextPage)
		return reply, nil
	}

	var t []string
	reply.List = make([]*v1.ManualTask, 0, len(list))
	for _, v := range list {
		for _, value := range strings.Split(v.TaskIds, ",") {
			tasks, err := model.NewTaskList().Get(database.DB().Where("task_id = ?", value))
			if err != nil && err != gorm.ErrRecordNotFound {
				return nil, err
			}
			if err == gorm.ErrRecordNotFound {
				return reply, nil
			}
			t = append(t, tasks.TaskName+":"+tasks.Desc)
		}

		var progress int64

		if v.Total != 0 {
			progress = int64(math.Floor((float64(v.FinishCount) / float64(v.Total)) * 100))
		} else {
			progress = 0
		}

		if v.IsFinish == 1 {
			progress = 100
		}

		reply.List = append(reply.List, &v1.ManualTask{
			ManualTaskName: v.TaskName,
			Source:         v.Source,
			TaskIds:        t,
			Total:          v.Total,
			FinishCount:    v.FinishCount,
			Progress:       progress,
			IsFinish:       v.IsFinish,
			CreatedAt:      time.Unix(v.CreatedAt, 0).Format("2006-01-02 15:04:05"),
			Type:           v.Type,
			Sql:            v.Sql,
		})
		t = []string{}
	}

	// 计算分页结果
	pagination.CalPage(int(count))

	reply.CurrentPage = int64(pagination.CurrentPage)
	reply.PerPage = int64(pagination.PerPage)
	reply.Total = int64(pagination.Total)
	reply.FirstPage = int64(pagination.FirstPage)
	reply.LastPage = int64(pagination.LastPage)
	reply.PrevPage = int64(pagination.PrevPage)
	reply.NextPage = int64(pagination.NextPage)
	return reply, nil
}
