package style_map_management

import (
	"adm-management/consts"
	"adm-management/errors"
	"adm-management/model"
	v1 "adm-management/pb/v1"
	"context"

	"encoding/json"
	"fmt"

	"git.getensh.com/common/gopkgsv2/database"
	"git.getensh.com/common/gopkgsv2/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
)

func StyleMap(ctx context.Context, req *v1.StyleMapRequest) (reply *v1.StyleMapReply, err error) {
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	reply = &v1.StyleMapReply{}

	db := database.DB()
	if req.Source == "" {
		db = db.Where("source is NOT null")
	}
	if req.Source != "" {
		db = db.Where("source = ?", req.Source)
	}

	if req.State == 1 {
		db = db.Where("style_id is NOT null and style_id != ''")
	} else if req.State == 2 {
		db = db.Where("style_id is null or style_id = ''")
	} else {
		db = db.Where("")
	}

	if req.ThirdStyleId == "" {
		db = db.Where("third_style_id is NOT null")
	}
	if req.ThirdStyleId != "" {
		db = db.Where("third_style_id = ?", req.ThirdStyleId)
	}

	if req.StyleId == "" {
		db = db.Where("")
	}
	if req.StyleId != "" {
		db = db.Where("style_id = ?", req.StyleId)
	}

	pageSize := consts.PageSize
	if req.PageSize != 0 {
		pageSize = int(req.PageSize)
	}

	// 构造分页类
	pagination := model.NewPagination(int(req.Page), pageSize, 0).GetLimitOffset()

	count, err := model.NewStyleMap().Count(db)
	if err != nil && err != gorm.ErrRecordNotFound {
		return reply, errors.SystemError
	}

	if err == gorm.ErrRecordNotFound || count == 0 {
		return reply, nil
	}

	list, err := model.NewStyleMap().List(db, pagination)

	if err != nil && err != gorm.ErrRecordNotFound {
		return reply, errors.SystemError
	}

	if err == gorm.ErrRecordNotFound {
		pagination.CalPage(int(count))

		reply.CurrentPage = int64(pagination.CurrentPage)
		reply.PerPage = int64(pagination.PerPage)
		reply.Total = int64(pagination.Total)
		reply.FirstPage = int64(pagination.FirstPage)
		reply.LastPage = int64(pagination.LastPage)
		reply.PrevPage = int64(pagination.PrevPage)
		reply.NextPage = int64(pagination.NextPage)
		return reply, nil
	}

	reply.List = make([]*v1.StyleMap, 0, len(list))
	var styleName string
	for _, v := range list {
		err = db.Raw("SELECT style_name FROM  db_adm_ads.t_adm_ads5 where style_id= ?", v.StyleId).Find(&styleName).Error
		if err != nil {
			if err == gorm.ErrRecordNotFound {
				return reply, nil
			}
			return reply, errors.SystemError
		}
		reply.List = append(reply.List, &v1.StyleMap{
			Source:          v.Source,
			ThirdBrandName:  v.ThirdBrandName,
			ThirdSeriesName: v.ThirdSeriesName,
			ThirdStyleName:  v.ThirdStyleName,
			ThirdStyleId:    v.ThirdStyleId,
			StyleId:         v.StyleId,
			StyleName:       styleName,
		})
		styleName = ""
	}

	// 计算分页结果
	pagination.CalPage(int(count))

	reply.CurrentPage = int64(pagination.CurrentPage)
	reply.PerPage = int64(pagination.PerPage)
	reply.Total = int64(pagination.Total)
	reply.FirstPage = int64(pagination.FirstPage)
	reply.LastPage = int64(pagination.LastPage)
	reply.PrevPage = int64(pagination.PrevPage)
	reply.NextPage = int64(pagination.NextPage)
	return reply, nil
}
