// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"encoding/json"
	"time"

	"git.getensh.com/common/gopkgsv2/logger"
	"go.uber.org/zap"
	"gorm.io/gorm"
)

type OfflineTask struct {
	ID          int64  `gorm:"primary_key" json:"-"`
	TaskName    string `gorm:"column:task_name"    json:"task_name"    form:"task_name"`
	Type        int64  `gorm:"column:type"         json:"type"         form:"type"`
	Source      string `gorm:"column:source"       json:"source"       form:"source"`
	Sql         string `gorm:"column:sql"          json:"sql"          form:"sql"`
	TaskIds     string `gorm:"column:task_ids"     json:"task_ids"     form:"task_ids"`
	Total       int64  `gorm:"column:total"        json:"total"        form:"total"`
	FinishCount int64  `gorm:"column:finish_count" json:"finish_count" form:"finish_count"`
	IsFinish    int64  `gorm:"column:is_finish"    json:"is_finish"    form:"is_finish"`
	CreatedAt   int64  `gorm:"column:created_at"   json:"created_at"   form:"created_at"`
	UpdatedAt   int64  `gorm:"column:updated_at"   json:"updated_at"   form:"updated_at"`
}

func (OfflineTask) TableName() string {
	return "t_adm_offline_task"
}

func MarshalJsonString(args interface{}) (result string) {
	if r, err := json.Marshal(args); err == nil {
		result = string(r)
	}
	return
}

// Insert 插入一条记录
func (p *OfflineTask) Insert(db *gorm.DB) error {
	timeNow := time.Now().Unix()
	p.UpdatedAt = timeNow
	p.CreatedAt = timeNow
	err := db.Create(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *OfflineTask) InsertByMap(db *gorm.DB, data map[string]interface{}) error {
	data["updated_at"] = time.Now().Unix()
	err := db.Model(p).Create(data).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", MarshalJsonString(data)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *OfflineTask) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "delete from "+p.TableName()),
			zap.String("fields", MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *OfflineTask) Save(db *gorm.DB) error {
	timeNow := time.Now().Unix()
	p.UpdatedAt = timeNow
	p.CreatedAt = timeNow
	//db.Model().Update()
	err := db.Save(p).Error
	if err != nil {

		logger.Error("mysql",
			zap.String("sql", "save "+p.TableName()),
			zap.String("fields", MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *OfflineTask) Update(db *gorm.DB) error {
	timeNow := time.Now().Unix()
	p.CreatedAt = 0
	p.UpdatedAt = timeNow
	err := db.Model(p).Updates(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", MarshalJsonString(*p)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *OfflineTask) UpdateByMap(db *gorm.DB, data map[string]interface{}) error {
	data["updated_at"] = time.Now().Unix()
	err := db.Model(p).Updates(data).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", MarshalJsonString(data)),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *OfflineTask) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	/*if filed == nil {
		return errors.ParamsError
	}*/
	timeNow := time.Now().Unix()
	filed["updated_at"] = timeNow
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", MarshalJsonString(filed)),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *OfflineTask) Query(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Find(p).Error
	if err != nil {

		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}

	if p.ID == 0 {
		return gorm.ErrRecordNotFound
	}

	return err
}

func (p *OfflineTask) QueryAll(db *gorm.DB, filter map[string]interface{}, out interface{}) error {
	err := db.Where(filter).Find(out).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", MarshalJsonString(filter)),
			zap.String("error", err.Error()))
	}
	return err
}
